function x = playtune(str,cb)
% Play musical tune.
% [x] = playtune(str,[cb])
%
% Notation:
% [CDEFGAB] keys, 5 full octaves
% [12345678] key/rest duration,
% with the default [1] semiquaver, [2] quaver, [4] crochet and [8] minim
% [#] sharp / [b] flat / [.] rest / [^] raise octave / [_] lower octave
%
% Examples:
% C major            -- playtune('^2CDEFGAB^C_BAGFEDC');
% F major            -- playtune('^2FGABb^CDEFEDC_BbAGF');
% B minor (harmonic) -- playtune('^2B^C#DEF#GA#BA#GF#EDC#_B');
% F# minor (melodic) -- playtune('^2F#G#AB^C#D#E#F#EDC#_BAG#F#');
% Doe-a-deer         -- playtune('^^6C2D6E2C4EC8E6D2EFFED8F.');
%
% Advanced features:
% CB is a (1,9) vector indicating the duration of that particular symbol (1-9).
%
% References:
% [1] Karplus & Strong, "Digital synthesis of plucked string and drum timbres", Computer Music Journal, vol. 7, no. 2, pp. 43-55, 1983.
% [2] Jaffe & Smith, "Extensions of the Karplus-Strong plucked-string algorithm", Computer Music Journal, vol. 7, no. 2, pp. 56-69, 1983.

if nargin < 2, cb = 1:8; end

% Set constants.
Fs = 8000;
Fc = 65.4064;
Td = 0.1;
F0 = power(2,(0:60)/12);

% Convert string to score.
d             = nan(1,length(str));
d(str == 'C') = 0;
d(str == 'D') = 2;
d(str == 'E') = 4;
d(str == 'F') = 5;
d(str == 'G') = 7;
d(str == 'A') = 9;
d(str == 'B') = 11;
d(str == '^') = 0;
d(str == '_') = 0;
d(str == '.') = 0;
u             = find(str == 'b');
d(u-1)        = d(u-1)-1;
str(u)        = [];
d(u)          = [];
u             = find(str == '#');
d(u-1)        = d(u-1)+1;
str(u)        = [];
d(u)          = [];

% Make music!
x  = [];
L  = round(cb(1)*Td*Fs);
oc = 1;
for k = 1:length(str)
   if isnan(d(k))
      L = round(cb(str2double(str(k)))*Td*Fs);
   else
      switch str(k)
         case '^',  oc           = oc+12;
         case '_',  oc           = oc-12;
         case '.',  x(end+L)     = 0;
         otherwise, x(end+(1:L)) = pluck1(Fc*F0(oc+d(k)),Fs,L); end
   end
end

if nargout > 0, return, end

soundsc(x,Fs);

% FUNCTIONS

function y = pluck1(F,Fs,L)

% Karplus-Strong [1]

p = round(Fs/F-0.5);
b = [1 1];
a = [2 zeros(1,p-1) -1 -1];
x = [sign(rand(1,2*p)-0.5) zeros(1,L-2*p)];
y = filter(b,a,x);

function y = pluck2(F,Fs,L)

% Extensions of Karplus-Strong [2]

p = floor(Fs/F);
d = Fs/F-p;
c = -(d-1)/(d+1);
b = [c c+1 1];
a = [2 2*c zeros(1,p-2) -c -c-1 -1];
x = [sign(rand(1,p)-0.5) zeros(1,L-p)];
y = filter(b,a,x);
